=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-03-13 20:18:05 +0000
+++ bzrlib/errors.py	2007-03-15 15:12:45 +0000
@@ -700,6 +700,16 @@
         self.obj = obj
 
 
+class ReadOnlyLockError(LockError):
+
+    _fmt = "Cannot acquire write lock on %(fname)s. %(msg)s"
+
+    def __init__(self, fname, msg):
+        LockError.__init__(self, '')
+        self.fname = fname
+        self.msg = msg
+
+
 class OutSideTransaction(BzrError):
 
     _fmt = ("A transaction related operation was attempted after"

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2007-03-01 21:56:19 +0000
+++ bzrlib/lock.py	2007-03-14 23:13:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
 It is not specified whether these locks are reentrant (i.e. can be
 taken repeatedly by a single process) or whether they exclude
-different threads in a single process.  That reentrancy is provided by 
+different threads in a single process.  That reentrancy is provided by
 LockableFiles.
 
 This defines two classes: ReadLock and WriteLock, which can be
@@ -35,60 +35,86 @@
 """
 
 import errno
-import os
 import sys
 
-from bzrlib.errors import LockError, LockContention
-from bzrlib.osutils import realpath
-from bzrlib.trace import mutter
+from bzrlib import (
+    errors,
+    osutils,
+    trace,
+    )
 
 
 class _base_Lock(object):
 
+    def __init__(self):
+        self.f = None
+        self.filename = None
+
     def _open(self, filename, filemode):
+        self.filename = osutils.realpath(filename)
         try:
-            self.f = open(filename, filemode)
+            self.f = open(self.filename, filemode)
             return self.f
         except IOError, e:
+            if e.errno in (errno.EACCES, errno.EPERM):
+                raise errors.ReadOnlyLockError(self.filename, str(e))
             if e.errno != errno.ENOENT:
                 raise
 
             # maybe this is an old branch (before may 2005)
-            mutter("trying to create missing branch lock %r", filename)
-            
-            self.f = open(filename, 'wb+')
+            trace.mutter("trying to create missing lock %r", self.filename)
+
+            self.f = open(self.filename, 'wb+')
             return self.f
 
+    def _clear_f(self):
+        """Clear the self.f attribute cleanly."""
+        if self.f:
+            self.f.close()
+            self.f = None
+
     def __del__(self):
         if self.f:
             from warnings import warn
             warn("lock on %r not released" % self.f)
             self.unlock()
-            
+
     def unlock(self):
         raise NotImplementedError()
 
 
-############################################################
-# msvcrt locks
-
-
 try:
     import fcntl
+    have_fcntl = True
+except ImportError:
+    have_fcntl = False
+try:
+    import win32con, win32file, pywintypes, winerror, msvcrt
+    have_pywin32 = True
+except ImportError:
+    have_pywin32 = False
+try:
+    import ctypes, msvcrt
+    have_ctypes = True
+except ImportError:
+    have_ctypes = False
+
+
+_lock_classes = []
+
+
+if have_fcntl:
+    LOCK_SH = fcntl.LOCK_SH
+    LOCK_NB = fcntl.LOCK_NB
+    lock_EX = fcntl.LOCK_EX
+
 
     class _fcntl_FileLock(_base_Lock):
 
-        f = None
-
         def _unlock(self):
             fcntl.lockf(self.f, fcntl.LOCK_UN)
             self._clear_f()
 
-        def _clear_f(self):
-            """Clear the self.f attribute cleanly."""
-            self.f.close()
-            del self.f
-
 
     class _fcntl_WriteLock(_fcntl_FileLock):
 
@@ -96,11 +122,11 @@
 
         def __init__(self, filename):
             # standard IO errors get exposed directly.
+            super(_fcntl_WriteLock, self).__init__()
             self._open(filename, 'rb+')
-            self.filename = realpath(filename)
             if self.filename in self.open_locks:
                 self._clear_f()
-                raise LockContention("Lock already held.")
+                raise errors.LockContention(self.filename)
             # reserve a slot for this lock - even if the lockf call fails,
             # at thisi point unlock() will be called, because self.f is set.
             # TODO: make this fully threadsafe, if we decide we care.
@@ -115,7 +141,7 @@
                     self.unlock()
                 # we should be more precise about whats a locking
                 # error and whats a random-other error
-                raise LockError(e)
+                raise errors.LockError(e)
 
         def unlock(self):
             del self.open_locks[self.filename]
@@ -124,8 +150,10 @@
 
     class _fcntl_ReadLock(_fcntl_FileLock):
 
+        open_locks = {}
+
         def __init__(self, filename):
-            # standard IO errors get exposed directly.
+            super(_fcntl_ReadLock, self).__init__()
             self._open(filename, 'rb')
             try:
                 # LOCK_NB will cause IOError to be raised if we can't grab a
@@ -134,153 +162,173 @@
             except IOError, e:
                 # we should be more precise about whats a locking
                 # error and whats a random-other error
-                raise LockError(e)
+                raise errors.LockError(e)
 
         def unlock(self):
             self._unlock()
 
 
-    WriteLock = _fcntl_WriteLock
-    ReadLock = _fcntl_ReadLock
-
-
-except ImportError:
-    try:
-        import win32con, win32file, pywintypes
-
-
-        LOCK_SH = 0 # the default
-        LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
-        LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
-
-        class _w32c_FileLock(_base_Lock):
-            def _lock(self, filename, openmode, lockmode):
-                try:
-                    self._open(filename, openmode)
-                    self.hfile = win32file._get_osfhandle(self.f.fileno())
-                    overlapped = pywintypes.OVERLAPPED()
-                    win32file.LockFileEx(self.hfile, lockmode, 0, 0x7fff0000, overlapped)
-                except Exception, e:
-                    if self.f:
-                        self.f.close()
-                        self.f = None
-                    raise LockError(e)
-
-            def unlock(self):
-                try:
-                    overlapped = pywintypes.OVERLAPPED()
-                    win32file.UnlockFileEx(self.hfile, 0, 0x7fff0000, overlapped)
-                    self.f.close()
-                    self.f = None
-                except Exception, e:
-                    raise LockError(e)
-
-
-        class _w32c_ReadLock(_w32c_FileLock):
-            def __init__(self, filename):
-                _w32c_FileLock._lock(self, filename, 'rb',
-                                     LOCK_NB)
-
-        class _w32c_WriteLock(_w32c_FileLock):
-            def __init__(self, filename):
-                _w32c_FileLock._lock(self, filename, 'rb+',
-                                     LOCK_EX + LOCK_NB)
-
-
-        WriteLock = _w32c_WriteLock
-        ReadLock = _w32c_ReadLock
-
-    except ImportError:
-        try:
-            import msvcrt
-
-
-            # Unfortunately, msvcrt.locking() doesn't distinguish between
-            # read locks and write locks. Also, the way the combinations
-            # work to get non-blocking is not the same, so we
-            # have to write extra special functions here.
-
-
-            class _msvc_FileLock(_base_Lock):
-                LOCK_SH = 1
-                LOCK_EX = 2
-                LOCK_NB = 4
-
-                def unlock(self):
-                    _msvc_unlock(self.f)
-                    self.f.close()
-                    self.f = None
-
-
-            class _msvc_ReadLock(_msvc_FileLock):
-                def __init__(self, filename):
-                    _msvc_lock(self._open(filename, 'rb'),
-                               self.LOCK_SH | self.LOCK_NB)
-
-
-            class _msvc_WriteLock(_msvc_FileLock):
-                def __init__(self, filename):
-                    _msvc_lock(self._open(filename, 'rb+'),
-                               self.LOCK_EX | self.LOCK_NB)
-
-
-            def _msvc_lock(f, flags):
-                try:
-                    # Unfortunately, msvcrt.LK_RLCK is equivalent to msvcrt.LK_LOCK
-                    # according to the comments, LK_RLCK is open the lock for writing.
-
-                    # Unfortunately, msvcrt.locking() also has the side effect that it
-                    # will only block for 10 seconds at most, and then it will throw an
-                    # exception, this isn't terrible, though.
-                    if type(f) == file:
-                        fpos = f.tell()
-                        fn = f.fileno()
-                        f.seek(0)
-                    else:
-                        fn = f
-                        fpos = os.lseek(fn, 0,0)
-                        os.lseek(fn, 0,0)
-
-                    if flags & _msvc_FileLock.LOCK_SH:
-                        if flags & _msvc_FileLock.LOCK_NB:
-                            lock_mode = msvcrt.LK_NBLCK
-                        else:
-                            lock_mode = msvcrt.LK_LOCK
-                    elif flags & _msvc_FileLock.LOCK_EX:
-                        if flags & _msvc_FileLock.LOCK_NB:
-                            lock_mode = msvcrt.LK_NBRLCK
-                        else:
-                            lock_mode = msvcrt.LK_RLCK
-                    else:
-                        raise ValueError('Invalid lock mode: %r' % flags)
-                    try:
-                        msvcrt.locking(fn, lock_mode, -1)
-                    finally:
-                        os.lseek(fn, fpos, 0)
-                except Exception, e:
-                    raise LockError(e)
-
-            def _msvc_unlock(f):
-                try:
-                    if type(f) == file:
-                        fpos = f.tell()
-                        fn = f.fileno()
-                        f.seek(0)
-                    else:
-                        fn = f
-                        fpos = os.lseek(fn, 0,0)
-                        os.lseek(fn, 0,0)
-
-                    try:
-                        msvcrt.locking(fn, msvcrt.LK_UNLCK, -1)
-                    finally:
-                        os.lseek(fn, fpos, 0)
-                except Exception, e:
-                    raise LockError(e)
-
-
-            WriteLock = _msvc_WriteLock
-            ReadLock = _msvc_ReadLock
-        except ImportError:
-            raise NotImplementedError("please write a locking method "
-                                      "for platform %r" % sys.platform)
+    _lock_classes.append(('fcntl', _fcntl_WriteLock, _fcntl_ReadLock))
+
+
+if have_pywin32 and sys.platform == 'win32':
+    LOCK_SH = 0 # the default
+    LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
+    LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
+
+
+    class _w32c_FileLock(_base_Lock):
+
+        def _lock(self, filename, openmode, lockmode):
+            self._open(filename, openmode)
+
+            self.hfile = msvcrt.get_osfhandle(self.f.fileno())
+            overlapped = pywintypes.OVERLAPPED()
+            try:
+                win32file.LockFileEx(self.hfile, lockmode, 0, 0x7fff0000,
+                                     overlapped)
+            except pywintypes.error, e:
+                self._clear_f()
+                if e.args[0] in (winerror.ERROR_LOCK_VIOLATION,):
+                    raise errors.LockContention(filename)
+                ## import pdb; pdb.set_trace()
+                raise
+            except Exception, e:
+                self._clear_f()
+                raise errors.LockError(e)
+
+        def unlock(self):
+            overlapped = pywintypes.OVERLAPPED()
+            try:
+                win32file.UnlockFileEx(self.hfile, 0, 0x7fff0000, overlapped)
+                self._clear_f()
+            except Exception, e:
+                raise errors.LockError(e)
+
+
+    class _w32c_ReadLock(_w32c_FileLock):
+        def __init__(self, filename):
+            super(_w32c_ReadLock, self).__init__()
+            self._lock(filename, 'rb', LOCK_SH + LOCK_NB)
+
+
+    class _w32c_WriteLock(_w32c_FileLock):
+        def __init__(self, filename):
+            super(_w32c_WriteLock, self).__init__()
+            self._lock(filename, 'rb+', LOCK_EX + LOCK_NB)
+
+
+    _lock_classes.append(('pywin32', _w32c_WriteLock, _w32c_ReadLock))
+
+
+if have_ctypes and sys.platform == 'win32':
+    # These constants were copied from the win32con.py module.
+    LOCKFILE_FAIL_IMMEDIATELY = 1
+    LOCKFILE_EXCLUSIVE_LOCK = 2
+    # Constant taken from winerror.py module
+    ERROR_LOCK_VIOLATION = 33
+
+    LOCK_SH = 0
+    LOCK_EX = LOCKFILE_EXCLUSIVE_LOCK
+    LOCK_NB = LOCKFILE_FAIL_IMMEDIATELY
+    _LockFileEx = ctypes.windll.kernel32.LockFileEx
+    _UnlockFileEx = ctypes.windll.kernel32.UnlockFileEx
+    _GetLastError = ctypes.windll.kernel32.GetLastError
+
+    ### Define the OVERLAPPED structure.
+    #   http://msdn2.microsoft.com/en-us/library/ms684342.aspx
+    # typedef struct _OVERLAPPED {
+    #   ULONG_PTR Internal;
+    #   ULONG_PTR InternalHigh;
+    #   union {
+    #     struct {
+    #       DWORD Offset;
+    #       DWORD OffsetHigh;
+    #     };
+    #     PVOID Pointer;
+    #   };
+    #   HANDLE hEvent;
+    # } OVERLAPPED,
+
+    class _inner_struct(ctypes.Structure):
+        _fields_ = [('Offset', ctypes.c_uint), # DWORD
+                    ('OffsetHigh', ctypes.c_uint), # DWORD
+                   ]
+
+    class _inner_union(ctypes.Union):
+        _fields_  = [('anon_struct', _inner_struct), # struct
+                     ('Pointer', ctypes.c_void_p), # PVOID
+                    ]
+
+    class OVERLAPPED(ctypes.Structure):
+        _fields_ = [('Internal', ctypes.c_void_p), # ULONG_PTR
+                    ('InternalHigh', ctypes.c_void_p), # ULONG_PTR
+                    ('_inner_union', _inner_union),
+                    ('hEvent', ctypes.c_void_p), # HANDLE
+                   ]
+
+    class _ctypes_FileLock(_base_Lock):
+
+        def _lock(self, filename, openmode, lockmode):
+            self._open(filename, openmode)
+
+            self.hfile = msvcrt.get_osfhandle(self.f.fileno())
+            overlapped = OVERLAPPED()
+            p_overlapped = ctypes.pointer(overlapped)
+            result = _LockFileEx(self.hfile, # HANDLE hFile
+                                 lockmode,   # DWORD dwFlags
+                                 0,          # DWORD dwReserved
+                                 0x7fffffff, # DWORD nNumberOfBytesToLockLow
+                                 0x00000000, # DWORD nNumberOfBytesToLockHigh
+                                 p_overlapped, # lpOverlapped
+                                )
+            if result == 0:
+                self._clear_f()
+                last_err = _GetLastError()
+                if last_err in (ERROR_LOCK_VIOLATION,):
+                    raise errors.LockContention(filename)
+                raise errors.LockError('Unknown locking error: %s'
+                                       % (last_err,))
+
+        def unlock(self):
+            overlapped = OVERLAPPED()
+            p_overlapped = ctypes.pointer(overlapped)
+            result = _UnlockFileEx(self.hfile, # HANDLE hFile
+                                   0,          # DWORD dwReserved
+                                   0x7fffffff, # DWORD nNumberOfBytesToLockLow
+                                   0x00000000, # DWORD nNumberOfBytesToLockHigh
+                                   p_overlapped, # lpOverlapped
+                                  )
+            self._clear_f()
+            if result == 0:
+                self._clear_f()
+                last_err = _GetLastError()
+                raise errors.LockError('Unknown unlocking error: %s'
+                                       % (last_err,))
+
+
+    class _ctypes_ReadLock(_ctypes_FileLock):
+        def __init__(self, filename):
+            super(_ctypes_ReadLock, self).__init__()
+            self._lock(filename, 'rb', LOCK_SH + LOCK_NB)
+
+
+    class _ctypes_WriteLock(_ctypes_FileLock):
+        def __init__(self, filename):
+            super(_ctypes_WriteLock, self).__init__()
+            self._lock(filename, 'rb+', LOCK_EX + LOCK_NB)
+
+
+    _lock_classes.append(('ctypes', _ctypes_WriteLock, _ctypes_ReadLock))
+
+
+if len(_lock_classes) == 0:
+    raise NotImplementedError(
+        "We must have one of fcntl, pywin32, or ctypes available"
+        " to support OS locking."
+        )
+
+
+# We default to using the first available lock class.
+_lock_type, WriteLock, ReadLock = _lock_classes[0]
+

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-03-13 21:14:39 +0000
+++ bzrlib/tests/__init__.py	2007-03-15 15:12:45 +0000
@@ -120,6 +120,7 @@
     import bzrlib.tests.interrepository_implementations
     import bzrlib.tests.interversionedfile_implementations
     import bzrlib.tests.intertree_implementations
+    import bzrlib.tests.per_lock
     import bzrlib.tests.repository_implementations
     import bzrlib.tests.revisionstore_implementations
     import bzrlib.tests.tree_implementations
@@ -132,6 +133,7 @@
             bzrlib.tests.interrepository_implementations,
             bzrlib.tests.interversionedfile_implementations,
             bzrlib.tests.intertree_implementations,
+            bzrlib.tests.per_lock,
             bzrlib.tests.repository_implementations,
             bzrlib.tests.revisionstore_implementations,
             bzrlib.tests.tree_implementations,

=== added directory 'bzrlib/tests/per_lock'
=== added file 'bzrlib/tests/per_lock/__init__.py'
--- bzrlib/tests/per_lock/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_lock/__init__.py	2007-03-14 20:15:52 +0000
@@ -0,0 +1,74 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""OS Lock implementation tests for bzr.
+
+These test the conformance of all the lock variations to the expected API.
+"""
+
+from copy import deepcopy
+
+from bzrlib import (
+    lock,
+    tests,
+    )
+
+
+class TestCaseWithLock(tests.TestCaseWithTransport):
+
+    write_lock = None
+    read_lock = None
+
+
+class LockTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple lock formats at once.
+
+    This is done by copying the test once for each lock and injecting the
+    read_lock and write_lock classes.
+    They are also given a new test id.
+    """
+
+    def __init__(self, lock_classes):
+        self._lock_classes = lock_classes
+
+    def _clone_test(self, test, write_lock, read_lock, variation):
+        """Clone test for adaption."""
+        new_test = deepcopy(test)
+        new_test.write_lock = write_lock
+        new_test.read_lock = read_lock
+        def make_new_test_id():
+            new_id = "%s(%s)" % (test.id(), variation)
+            return lambda: new_id
+        new_test.id = make_new_test_id()
+        return new_test
+
+    def adapt(self, test):
+        result = tests.TestSuite()
+        for name, write_lock, read_lock in self._lock_classes:
+            new_test = self._clone_test(test, write_lock, read_lock, name)
+            result.addTest(new_test)
+        return result
+
+
+def test_suite():
+    result = tests.TestSuite()
+    test_lock_implementations = [
+        'bzrlib.tests.per_lock.test_lock',
+        ]
+    adapter = LockTestProviderAdapter(lock._lock_classes)
+    loader = tests.TestLoader()
+    tests.adapt_modules(test_lock_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/per_lock/test_lock.py'
--- bzrlib/tests/per_lock/test_lock.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_lock/test_lock.py	2007-03-14 20:15:52 +0000
@@ -0,0 +1,118 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for OS level locks."""
+
+from bzrlib import (
+    errors,
+    osutils,
+    )
+
+from bzrlib.tests.per_lock import TestCaseWithLock
+
+
+class TestLock(TestCaseWithLock):
+
+    def setUp(self):
+        super(TestLock, self).setUp()
+        self.build_tree(['a-file'])
+
+    def test_read_lock(self):
+        """Smoke test for read locks."""
+        a_lock = self.read_lock('a-file')
+        self.addCleanup(a_lock.unlock)
+        # The lock file should be opened for reading
+        txt = a_lock.f.read()
+        self.assertEqual('contents of a-file\n', txt)
+
+    def test_create_if_needed_read(self):
+        """We will create the file if it doesn't exist yet."""
+        a_lock = self.read_lock('other-file')
+        self.addCleanup(a_lock.unlock)
+        txt = a_lock.f.read()
+        self.assertEqual('', txt)
+
+    def test_create_if_needed_write(self):
+        """We will create the file if it doesn't exist yet."""
+        a_lock = self.write_lock('other-file')
+        self.addCleanup(a_lock.unlock)
+        txt = a_lock.f.read()
+        self.assertEqual('', txt)
+        a_lock.f.write('foo\n')
+        a_lock.f.seek(0)
+        txt = a_lock.f.read()
+        self.assertEqual('foo\n', txt)
+
+    def test_readonly_file(self):
+        """If the file is readonly, we can take a read lock.
+
+        But we shouldn't be able to take a write lock.
+        """
+        osutils.make_readonly('a-file')
+        # Make sure the file is read-only (on all platforms)
+        self.assertRaises(IOError, open, 'a-file', 'rb+')
+        a_lock = self.read_lock('a-file')
+        a_lock.unlock()
+
+        # TODO: jam 20070313 This should be a specific subclass
+        self.assertRaises(errors.ReadOnlyLockError, self.write_lock, 'a-file')
+
+    def test_write_lock(self):
+        """Smoke test for write locks."""
+        a_lock = self.write_lock('a-file')
+        self.addCleanup(a_lock.unlock)
+        # You should be able to read and write to the lock file.
+        txt = a_lock.f.read()
+        self.assertEqual('contents of a-file\n', txt)
+        # Win32 requires that you call seek() when switching between a read
+        # operation and a write operation.
+        a_lock.f.seek(0, 2)
+        a_lock.f.write('more content\n')
+        a_lock.f.seek(0)
+        txt = a_lock.f.read()
+        self.assertEqual('contents of a-file\nmore content\n', txt)
+
+    def test_multiple_read_locks(self):
+        """You can take out more than one read lock on the same file."""
+        a_lock = self.read_lock('a-file')
+        self.addCleanup(a_lock.unlock)
+        b_lock = self.read_lock('a-file')
+        self.addCleanup(b_lock.unlock)
+
+    def test_multiple_write_locks_exclude(self):
+        """Taking out more than one write lock should fail."""
+        a_lock = self.write_lock('a-file')
+        self.addCleanup(a_lock.unlock)
+        # Taking out a lock on a locked file should raise LockContention
+        self.assertRaises(errors.LockContention, self.write_lock, 'a-file')
+
+    def _disabled_test_read_then_write_excludes(self):
+        """If a file is read-locked, taking out a write lock should fail."""
+        a_lock = self.read_lock('a-file')
+        self.addCleanup(a_lock.unlock)
+        # Taking out a lock on a locked file should raise LockContention
+        self.assertRaises(errors.LockContention, self.write_lock, 'a-file')
+
+    def _disabled_test_write_then_read_excludes(self):
+        """If a file is write-locked, taking out a read lock should fail.
+
+        The file is exclusively owned by the write lock, so we shouldn't be
+        able to take out a shared read lock.
+        """
+        a_lock = self.write_lock('a-file')
+        self.addCleanup(a_lock.unlock)
+        # Taking out a lock on a locked file should raise LockContention
+        self.assertRaises(errors.LockContention, self.read_lock, 'a-file')

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2007-02-26 22:08:14 +0000
+++ bzrlib/tests/test_errors.py	2007-03-14 20:47:17 +0000
@@ -105,6 +105,11 @@
             "to be.",
             str(error))
 
+    def test_read_only_lock_error(self):
+        error = errors.ReadOnlyLockError('filename', 'error message')
+        self.assertEqualDiff("Cannot acquire write lock on filename."
+                             " error message", str(error))
+
     def test_too_many_concurrent_requests(self):
         error = errors.TooManyConcurrentRequests("a medium")
         self.assertEqualDiff("The medium 'a medium' has reached its concurrent "

